@@ -105,6 +105,13 @@ subroutine calcbz_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
       end do
       call rchkusr
     end do
+  case (7)
+    do j = 1, Ntot
+      do k = 1, n_nu
+        logfy(k, j) = condyz_bw(n, y, l, zsample(:, j), nu(k), tsq)
+      end do
+      call rchkusr
+    end do
   case default
     call rexit ("Unrecognised family")
   end select
@@ -125,6 +132,76 @@ subroutine calcbz_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
 end subroutine calcbz_st
 
 
+subroutine calcbw_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
+   Ntot, wsample, weights, n, p, betm0, betQ0, ssqdf, &
+   ssqsc, tsqdf, tsq, y, l, F, dm, ifam)
+
+  !use interfaces
+  use flogsumexp
+  use covfun
+  !use pdfz
+  !use condyz, only: condyz_bi
+  use transfbinomial
+  implicit none
+  integer, intent(in) :: n, p, Ntot, n_cov, n_nu, ifam, icf
+  double precision, intent(in) :: phi(n_cov), nsq(n_cov), kappa(n_cov), &
+     nu(n_nu), wsample(n,Ntot), weights(Ntot), &
+     betm0(p), betQ0(p,p), ssqdf, ssqsc, tsqdf, tsq, y(n), l(n), &
+     F(n,p), dm(n,n)
+  double precision, intent(out) :: bfact(n_nu,n_cov)
+  logical lup(n, n), lmxi
+  double precision ssqdfsc, tsqdfsc, respdfh, modeldfh
+  integer i, j, k
+  double precision T(n,n), TiF(n,p), FTF(p,p), Ups(n,n), &
+     ldh_Ups, llikw(n_nu,Ntot), xi(n), lfw
+
+  ssqdfsc = ssqdf*ssqsc
+  tsqdfsc = tsqdf*tsq
+  respdfh = .5d0*(n + tsqdf)
+
+  ! Determine flat or normal prior
+  j=0
+  do i = 1, p
+    if (betQ0(i,i) /= 0d0) j = j + 1
+  end do
+  if (j == 0) then ! Flat prior
+    modeldfh = .5d0*(n - p + ssqdf)
+    lmxi = .false.
+  else ! Normal prior
+    modeldfh = .5d0*(n + ssqdf)
+    xi = matmul(F,betm0)
+    lmxi = any(xi .ne. 0d0)
+  end if
+
+  do i = 1, n
+    lup(:i-1,i) = .true.
+    lup(i:,i) = .false. 
+  end do
+
+  call rchkusr
+
+  
+  select case (ifam)
+  case (2)
+    do i = 1, n_cov
+      call rchkusr
+      call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
+         lup,kappa(i),icf,n,p,T,TiF,FTF,Ups,ldh_Ups)
+      do j = 1, Ntot
+        do k = 1, n_nu
+          lfw = jointyw_bi(n, wsample(:, j), y, l, Ups, ldh_Ups, &
+             nu(k), xi, lmxi, ssqdfsc, tsq, modeldfh)
+          llikw(k, j) = lfw - weights(j)
+        end do
+      end do
+      bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
+    end do
+  case default
+    call rexit ("Unrecognised family")
+  end select
+end subroutine calcbw_st
+
+
 subroutine calcbz_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
    Ntot, zsample, weights, zcv, n, p, kg, betm0, betQ0, ssqdf, &
    ssqsc, tsqdf, tsq, y, l, F, dm, ifam)
@@ -144,8 +221,8 @@ subroutine calcbz_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
   logical lup(n, n), lmxi
   double precision ssqdfsc, tsqdfsc, respdfh, modeldfh
   integer i, j, k
-  double precision, allocatable :: tmp(:)
   integer ltmp
+  double precision tmp(32*kg)
   double precision qrtau(kg), zcvqr(Ntot, kg), logfy(n_nu, Ntot), lfz, dNtot
   double precision T(n, n), TiF(n, p), FTF(p, p), Ups(n, n), &
      ldh_Ups, ycv(n_nu, Ntot), llikw, xi(n)
@@ -228,21 +305,20 @@ subroutine calcbz_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
       end do
       call rchkusr
     end do
+  case (7)
+    do j = 1, Ntot
+      do k = 1, n_nu
+        logfy(k, j) = condyz_bw(n, y, l, zsample(:, j), nu(k), tsq)
+      end do
+      call rchkusr
+    end do
   case default
     call rexit ("Unrecognised family")
   end select
 
   ! QR factorisation
   zcvqr = zcv
-  ! Determine optimal work size for QR factorisation.
-  ltmp = -1
-  call dgeqrf (Ntot, kg, zcvqr, Ntot, qrtau, lfz, ltmp, i)
-  if (i .ne. 0) then
-    call rexit ('calcb - Cannot determine optimal space for regression')
-  else
-    ltmp = nint(lfz)
-    allocate (tmp(ltmp))
-  end if
+  ltmp = size(tmp)
   call dgeqrf (Ntot, kg, zcvqr, Ntot, qrtau, tmp, ltmp, i)
   if (i .ne. 0) then
     call rexit ('calcb - Error with the QR factorisation')
@@ -260,24 +336,92 @@ subroutine calcbz_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         ycv(i, j) = exp(llikw + dNtot)
       end do
     end do
-
-    ! Regression
-    call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,i)
-    if (i .ne. 0) then
-      call rexit ('calcb - Error in DORMQR')
-    end if
-    call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-    where (ycv(:, 1) > 0d0)
-      bfact(:,k) = log(ycv(:, 1))
-    elsewhere
-      bfact(:,k) = -huge(1d0)
-    end where
+    call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,k))
   end do
-  deallocate(tmp)
 end subroutine calcbz_cv
 
 
 
+
+subroutine calcbw_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
+   Ntot, wsample, weights, zcv, n, p, kg, betm0, betQ0, ssqdf, &
+   ssqsc, tsqdf, tsq, y, l, F, dm, ifam)
+
+  use flogsumexp
+  use covfun
+  use transfbinomial
+  implicit none
+  integer, intent(in) :: n, p, kg, Ntot, n_cov, n_nu, ifam, icf
+  double precision, intent(in) :: phi(n_cov), nsq(n_cov), kappa(n_cov), &
+     nu(n_nu), wsample(n,Ntot), weights(Ntot), zcv(Ntot, kg), &
+     betm0(p), betQ0(p,p), ssqdf, ssqsc, tsqdf, tsq, y(n), l(n), &
+     F(n,p), dm(n,n)
+  double precision, intent(out) :: bfact(n_nu,n_cov)
+  logical lup(n, n), lmxi
+  double precision ssqdfsc, tsqdfsc, respdfh, modeldfh
+  integer i, j, k
+  double precision T(n,n), TiF(n,p), FTF(p,p), Ups(n,n), &
+     ldh_Ups, llikw, ycv(n_nu,Ntot), xi(n), lfw
+  integer ltmp
+  double precision qrtau(kg), zcvqr(Ntot, kg), dNtot, tmp(32*kg)
+
+  ssqdfsc = ssqdf*ssqsc
+  tsqdfsc = tsqdf*tsq
+  respdfh = .5d0*(n + tsqdf)
+  dNtot = log(dble(Ntot))
+
+  ! Determine flat or normal prior
+  j=0
+  do i = 1, p
+    if (betQ0(i,i) /= 0d0) j = j + 1
+  end do
+  if (j == 0) then ! Flat prior
+    modeldfh = .5d0*(n - p + ssqdf)
+    lmxi = .false.
+  else ! Normal prior
+    modeldfh = .5d0*(n + ssqdf)
+    xi = matmul(F,betm0)
+    lmxi = any(xi .ne. 0d0)
+  end if
+
+  do i = 1, n
+    lup(:i-1,i) = .true.
+    lup(i:,i) = .false. 
+  end do
+
+  call rchkusr
+
+  ! QR factorisation
+  zcvqr = zcv
+  ltmp = size(tmp)
+  call dgeqrf (Ntot, kg, zcvqr, Ntot, qrtau, tmp, ltmp, i)
+  if (i .ne. 0) then
+    call rexit ('calcb - Error with the QR factorisation')
+  end if
+
+  select case (ifam)
+  case (2)
+    do i = 1, n_cov
+      call rchkusr
+      call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
+         lup,kappa(i),icf,n,p,T,TiF,FTF,Ups,ldh_Ups)
+      do j = 1, Ntot
+        do k = 1, n_nu
+          lfw = jointyw_bi(n, wsample(:, j), y, l, Ups, ldh_Ups, &
+             nu(k), xi, lmxi, ssqdfsc, tsq, modeldfh)
+          llikw = lfw - weights(j)
+          ycv(k,j) = exp(llikw + dNtot)
+        end do
+      end do
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
+    end do
+  case default
+    call rexit ("Unrecognised family")
+  end select
+end subroutine calcbw_cv
+
+
+
 subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
    Ntot, musample, weights, n, p, betm0, betQ0, ssqdf, &
    ssqsc, tsqdf, tsq, y, l, F, dm, ifam)
@@ -287,6 +431,7 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
   use covfun
   use condymu
   use pdfmu
+  use transfbinomial
   implicit none
   integer, intent(in) :: n, p, Ntot, n_cov, n_nu, ifam, icf
   double precision, intent(in) :: phi(n_cov), nsq(n_cov), kappa(n_cov), &
@@ -328,9 +473,6 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
 
   select case (ifam)
   case (0)
-    do j = 1, Ntot
-      logfy(j) = condymu_gt(n, y, l, musample(:, j), tsqdfsc, respdfh)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -339,15 +481,12 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_ga(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw(k, j) = logfy(j) + lfmu - weights(j)
+          llikw(k, j) = lfmu - weights(j)
         end do
       end do
       bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
     end do
   case (1)
-    do j = 1, Ntot
-      logfy(j) = condymu_ga(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -356,15 +495,12 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_ga(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw(k, j) = logfy(j) + lfmu - weights(j)
+          llikw(k, j) = lfmu - weights(j)
         end do
       end do
       bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
     end do
   case (2)
-    do j = 1, Ntot
-      logfy(j) = condymu_bi(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -373,15 +509,12 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_bi(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw(k, j) = logfy(j) + lfmu - weights(j)
+          llikw(k, j) = lfmu - weights(j)
         end do
       end do
       bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
     end do
   case (3)
-    do j = 1, Ntot
-      logfy(j) = condymu_po(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -390,15 +523,12 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_po(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw(k, j) = logfy(j) + lfmu - weights(j)
+          llikw(k, j) = lfmu - weights(j)
         end do
       end do
       bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
     end do
   case (4)
-    do j = 1, Ntot
-      logfy(j) = condymu_gm(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -407,15 +537,12 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_gm(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw(k, j) = logfy(j) + lfmu - weights(j)
+          llikw(k, j) = lfmu - weights(j)
         end do
       end do
       bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
     end do
   case (5)
-    do j = 1, Ntot
-      logfy(j) = condymu_ba(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -424,15 +551,12 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_ba(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw(k, j) = logfy(j) + lfmu - weights(j)
+          llikw(k, j) = lfmu - weights(j)
         end do
       end do
       bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
     end do
   case (6)
-    do j = 1, Ntot
-      logfy(j) = condymu_bd(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -441,7 +565,21 @@ subroutine calcbmu_st (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_bd(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw(k, j) = logfy(j) + lfmu - weights(j)
+          llikw(k, j) = lfmu - weights(j)
+        end do
+      end do
+      bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
+    end do
+  case (7)
+    do i = 1, n_cov
+      call rchkusr
+      call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
+         lup,kappa(i),icf,n,p,T,TiF,FTF,Ups,ldh_Ups)
+      do j = 1, Ntot
+        do k = 1, n_nu
+          lfmu = logpdfmu_bw(n, musample(:, j), Ups, ldh_Ups, &
+             nu(k), xi, lmxi, ssqdfsc, modeldfh)
+          llikw(k, j) = lfmu - weights(j)
         end do
       end do
       bfact(:, i) = logrsumexp(llikw, n_nu, Ntot)
@@ -471,9 +609,9 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
   logical lup(n, n), lmxi
   double precision ssqdfsc, tsqdfsc, respdfh, modeldfh
   integer i, j, k
-  double precision, allocatable :: tmp(:)
   integer ltmp
-  double precision qrtau(kg), zcvqr(Ntot, kg), logfy(Ntot), lfmu, dNtot
+  double precision :: tmp(32*kg)
+  double precision qrtau(kg), zcvqr(Ntot, kg), lfmu, dNtot
   double precision T(n, n), TiF(n, p), FTF(p, p), Ups(n, n), &
      ldh_Ups, ycv(n_nu, Ntot), llikw, xi(n)
 
@@ -505,15 +643,7 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
 
   ! QR factorisation
   zcvqr = zcv
-  ! Determine optimal work size for QR factorisation.
-  ltmp = -1
-  call dgeqrf (Ntot, kg, zcvqr, Ntot, qrtau, lfmu, ltmp, i)
-  if (i .ne. 0) then
-    call rexit ('calcb - Cannot determine optimal space for regression')
-  else
-    ltmp = nint(lfmu)
-    allocate (tmp(ltmp))
-  end if
+  ltmp = size(tmp)
   call dgeqrf (Ntot, kg, zcvqr, Ntot, qrtau, tmp, ltmp, i)
   if (i .ne. 0) then
     call rexit ('calcb - Error with the QR factorisation')
@@ -521,9 +651,6 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
 
   select case (ifam)
   case (0)
-    do j = 1, Ntot
-      logfy(j) = condymu_gt(n, y, l, musample(:, j), tsqdfsc, respdfh)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -532,26 +659,13 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_ga(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw = logfy(j) + lfmu - weights(j)
+          llikw = lfmu - weights(j)
           ycv(k, j) = exp(llikw + dNtot)
         end do
       end do
-      ! Regression
-      call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,j)
-      if (j .ne. 0) then
-        call rexit ('calcb - Error in DORMQR')
-      end if
-      call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-      where (ycv(:, 1) > 0d0)
-        bfact(:,i) = log(ycv(:, 1))
-      elsewhere
-        bfact(:,i) = -huge(1d0)
-      end where
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
     end do
   case (1)
-    do j = 1, Ntot
-      logfy(j) = condymu_ga(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -560,26 +674,13 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_ga(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw = logfy(j) + lfmu - weights(j)
+          llikw = lfmu - weights(j)
           ycv(k, j) = exp(llikw + dNtot)
         end do
       end do
-      ! Regression
-      call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,j)
-      if (j .ne. 0) then
-        call rexit ('calcb - Error in DORMQR')
-      end if
-      call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-      where (ycv(:, 1) > 0d0)
-        bfact(:,i) = log(ycv(:, 1))
-      elsewhere
-        bfact(:,i) = -huge(1d0)
-      end where
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
     end do
   case (2)
-    do j = 1, Ntot
-      logfy(j) = condymu_bi(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -588,26 +689,13 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_bi(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw = logfy(j) + lfmu - weights(j)
+          llikw = lfmu - weights(j)
           ycv(k, j) = exp(llikw + dNtot)
         end do
       end do
-      ! Regression
-      call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,j)
-      if (j .ne. 0) then
-        call rexit ('calcb - Error in DORMQR')
-      end if
-      call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-      where (ycv(:, 1) > 0d0)
-        bfact(:,i) = log(ycv(:, 1))
-      elsewhere
-        bfact(:,i) = -huge(1d0)
-      end where
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
     end do
   case (3)
-    do j = 1, Ntot
-      logfy(j) = condymu_po(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -616,26 +704,13 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_po(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw = logfy(j) + lfmu - weights(j)
+          llikw = lfmu - weights(j)
           ycv(k, j) = exp(llikw + dNtot)
         end do
       end do
-      ! Regression
-      call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,j)
-      if (j .ne. 0) then
-        call rexit ('calcb - Error in DORMQR')
-      end if
-      call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-      where (ycv(:, 1) > 0d0)
-        bfact(:,i) = log(ycv(:, 1))
-      elsewhere
-        bfact(:,i) = -huge(1d0)
-      end where
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
     end do
   case (4)
-    do j = 1, Ntot
-      logfy(j) = condymu_gm(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -644,26 +719,13 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_gm(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw = logfy(j) + lfmu - weights(j)
+          llikw = lfmu - weights(j)
           ycv(k, j) = exp(llikw + dNtot)
         end do
       end do
-      ! Regression
-      call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,j)
-      if (j .ne. 0) then
-        call rexit ('calcb - Error in DORMQR')
-      end if
-      call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-      where (ycv(:, 1) > 0d0)
-        bfact(:,i) = log(ycv(:, 1))
-      elsewhere
-        bfact(:,i) = -huge(1d0)
-      end where
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
     end do
   case (5)
-    do j = 1, Ntot
-      logfy(j) = condymu_ba(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -672,26 +734,13 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_ba(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw = logfy(j) + lfmu - weights(j)
+          llikw = lfmu - weights(j)
           ycv(k, j) = exp(llikw + dNtot)
         end do
       end do
-      ! Regression
-      call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,j)
-      if (j .ne. 0) then
-        call rexit ('calcb - Error in DORMQR')
-      end if
-      call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-      where (ycv(:, 1) > 0d0)
-        bfact(:,i) = log(ycv(:, 1))
-      elsewhere
-        bfact(:,i) = -huge(1d0)
-      end where
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
     end do
   case (6)
-    do j = 1, Ntot
-      logfy(j) = condymu_bd(n, y, l, musample(:, j), tsq)
-    end do
     do i = 1, n_cov
       call rchkusr
       call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
@@ -700,26 +749,55 @@ subroutine calcbmu_cv (bfact, phi, nu, nsq, kappa, icf, n_cov, n_nu, &
         do k = 1, n_nu
           lfmu = logpdfmu_bd(n, musample(:, j), Ups, ldh_Ups, &
              nu(k), xi, lmxi, ssqdfsc, modeldfh)
-          llikw = logfy(j) + lfmu - weights(j)
+          llikw = lfmu - weights(j)
           ycv(k, j) = exp(llikw + dNtot)
         end do
       end do
-      ! Regression
-      call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,j)
-      if (j .ne. 0) then
-        call rexit ('calcb - Error in DORMQR')
-      end if
-      call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
-      where (ycv(:, 1) > 0d0)
-        bfact(:,i) = log(ycv(:, 1))
-      elsewhere
-        bfact(:,i) = -huge(1d0)
-      end where
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
+    end do
+  case (7)
+    do i = 1, n_cov
+      call rchkusr
+      call calc_cov (phi(i),nsq(i),dm,F,betQ0,&
+         lup,kappa(i),icf,n,p,T,TiF,FTF,Ups,ldh_Ups)
+      do j = 1, Ntot
+        do k = 1, n_nu
+          lfmu = logpdfmu_bw(n, musample(:, j), Ups, ldh_Ups, &
+             nu(k), xi, lmxi, ssqdfsc, modeldfh)
+          llikw = lfmu - weights(j)
+          ycv(k, j) = exp(llikw + dNtot)
+        end do
+      end do
+      call calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact(:,i))
     end do
   case default
     call rexit ("Unrecognised family")
   end select
-
-  deallocate(tmp)
 end subroutine calcbmu_cv
 
+
+
+
+subroutine calcbfcv (n_nu,Ntot,kg,zcvqr,qrtau,ycv,bfact)
+  implicit none
+  integer, intent(in) :: n_nu, Ntot, kg
+  double precision, intent(in) :: zcvqr(Ntot,kg), qrtau(kg)
+  double precision, intent(inout) :: ycv(n_nu,Ntot)
+  double precision, intent(out) :: bfact(n_nu)
+  integer ltmp, info
+  double precision tmp(n_nu*32)
+
+  ltmp = size(tmp)
+  call dormqr ('r','n',n_nu,Ntot,kg,zcvqr,Ntot,qrtau,ycv,n_nu,tmp,ltmp,&
+     info) 
+  if (info .ne. 0) then
+    call rexit ('calcb - Error in DORMQR')
+  end if
+  call dtrsm ('r','u','t','n',n_nu,kg,1d0,zcvqr,Ntot,ycv,n_nu)
+  where (ycv(:, 1) .gt. 0d0)
+    bfact = log(ycv(:, 1))
+  elsewhere
+    bfact = -huge(1d0)
+  end where
+end subroutine calcbfcv
+
